/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.sg.balancedlibrary.materialReportDetails.service;

import com.jeeplus.common.json.AjaxJson;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.fieldtype.AreaType;
import com.jeeplus.core.persistence.DataEntity;
import com.jeeplus.core.persistence.Page;
import com.jeeplus.core.service.CrudService;
import com.jeeplus.modules.sg.balancedlibrary.liKuResourcePool.entity.LiKuResourcePool;
import com.jeeplus.modules.sg.balancedlibrary.liKuResourcePool.mapper.LiKuResourcePoolMapper;
import com.jeeplus.modules.sg.balancedlibrary.materialReportDetails.constant.AreaCodeFlagEnum;
import com.jeeplus.modules.sg.balancedlibrary.materialReportDetails.constant.AreaTypeFlagEnum;
import com.jeeplus.modules.sg.balancedlibrary.materialReportDetails.constant.MaterialsUnitFlagEnum;
import com.jeeplus.modules.sg.balancedlibrary.materialReportDetails.entity.LikuDetail;
import com.jeeplus.modules.sg.balancedlibrary.materialReportDetails.entity.MaterialReportDetails;
import com.jeeplus.modules.sg.balancedlibrary.materialReportDetails.mapper.MaterialReportDetailsMapper;
import com.jeeplus.modules.sg.balancedlibrary.onPassageMaterials.entity.MaterialInformation;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * 物资上报明细Service
 *
 * @author enford
 * @version 1.0
 */
@Service
@Transactional(readOnly = true)
public class MaterialReportDetailsService extends CrudService<MaterialReportDetailsMapper, MaterialReportDetails> {

    @Autowired
    private LiKuResourcePoolMapper liKuResourcePoolMapper;
    @Autowired
    private MaterialReportDetailsMapper materialReportDetailsMapper;

    @Override
    public MaterialReportDetails get(String id) {
        return super.get(id);
    }

    @Override
    public List<MaterialReportDetails> findList(MaterialReportDetails entity) {
        return super.findList(entity);
    }

    @Override
    public Page<MaterialReportDetails> findPage(Page<MaterialReportDetails> page, MaterialReportDetails entity) {
        return super.findPage(page, entity);
    }

    @Override
    @Transactional(readOnly = false)
    public void save(MaterialReportDetails entity) {
        super.save(entity);
    }

    @Override
    @Transactional(readOnly = false)
    public void delete(MaterialReportDetails entity) {
        super.delete(entity);
    }

    @Transactional(readOnly = false)
    public void insertList(List<MaterialReportDetails> list) {
        mapper.insertList(list);
    }

    @Transactional(readOnly = false)
    public void delAll() {
        mapper.delAll();
    }

    public List<LikuDetail> getLikuDetails(MaterialReportDetails report) {
        return mapper.getLikuDetails(report);
    }

    @Transactional(readOnly = false)
    public AjaxJson liKuBalance(List<MaterialReportDetails> materialReportDetailsList) {
        AjaxJson j = new AjaxJson();
        /*************************************对利库资源池进行操作**********************************************/
        //段长利库池
        List<LiKuResourcePool> liKuIsSegmentPoolList = new ArrayList<>();
        //非段长利库池
        List<LiKuResourcePool> liKuIsNotSegmentPoolList = new ArrayList<>();

        List<LiKuResourcePool> liKuPoolList = liKuResourcePoolMapper.findLiKuList(null, null);
        //如果利库资源池为空时 则直接返回结果提示
        if (liKuPoolList == null || liKuPoolList.size() == 0) {
            j.setSuccess(false);
            j.setMsg("请先导入利库资源池！");
            return j;
        }


        //循环初始化
        liKuPoolList.forEach(liKuResourcePool -> {
            //初始化剩余数量
            liKuResourcePool.setRemaining(liKuResourcePool.getInventoryAmount());
            //初始化利库数量
            liKuResourcePool.setNumberOfNiches("0");


            //添加枚举标识
            if (liKuResourcePool.getInvLocation().equals(AreaCodeFlagEnum.HLA2.getCode())) {
                liKuResourcePool.setLiKuFlag(AreaCodeFlagEnum.HLA2);
            } else if (liKuResourcePool.getInvLocation().equals(AreaCodeFlagEnum.HLB1.getCode())) {
                liKuResourcePool.setLiKuFlag(AreaCodeFlagEnum.HLB1);
            } else if (liKuResourcePool.getInvLocation().equals(AreaCodeFlagEnum.HLC1.getCode())) {
                liKuResourcePool.setLiKuFlag(AreaCodeFlagEnum.HLC1);
            } else if (liKuResourcePool.getInvLocation().equals(AreaCodeFlagEnum.HLD1.getCode())) {
                liKuResourcePool.setLiKuFlag(AreaCodeFlagEnum.HLD1);
            } else {
                liKuResourcePool.setLiKuFlag(AreaCodeFlagEnum.NO_PLACE);
            }

            //添加标识 分组 段长数据 非段长数据 单位转换
            if (liKuResourcePool.getUnitOfMeasurement().equals(MaterialsUnitFlagEnum.M.getUnit()) || liKuResourcePool.getUnitOfMeasurement().equals(MaterialsUnitFlagEnum.KM.getUnit())) {
                if (StringUtils.isNotBlank(liKuResourcePool.getLengthOfCable())) {
                    String[] split = liKuResourcePool.getLengthOfCable().split(";");
                    for (int i = 0; i < split.length; i++) {
                        //拆分成新的对象添加集合中 拆分段长为剩余
                        LiKuResourcePool ResourcePool = new LiKuResourcePool();
                        if (liKuResourcePool.getUnitOfMeasurement().equals(MaterialsUnitFlagEnum.KM.getUnit())) {
                            BeanUtils.copyProperties(liKuResourcePool, ResourcePool);
                            ResourcePool.setUnitOfMeasurement(MaterialsUnitFlagEnum.M.getUnit());
                            ResourcePool.setInventoryAmount(new BigDecimal(liKuResourcePool.getInventoryAmount()).multiply(new BigDecimal(1000)).toString());
                            ResourcePool.setRemaining(liKuResourcePool.getInventoryAmount());
                            ResourcePool.setRemaining(new BigDecimal(split[i]).multiply(new BigDecimal(1000)).toString());
                            ResourcePool.setLengthOfCable(new BigDecimal(split[i]).multiply(new BigDecimal(1000)).toString());
                        } else {
                            liKuResourcePool.setRemaining(split[i]);
                            liKuResourcePool.setLengthOfCable(split[i]);
                            BeanUtils.copyProperties(liKuResourcePool, ResourcePool);
                        }

                        liKuIsSegmentPoolList.add(ResourcePool);
                    }
                } else {
                    if (liKuResourcePool.getUnitOfMeasurement().equals(MaterialsUnitFlagEnum.KM.getUnit())) {
                        liKuResourcePool.setUnitOfMeasurement(MaterialsUnitFlagEnum.M.getUnit());
                        liKuResourcePool.setInventoryAmount(new BigDecimal(liKuResourcePool.getInventoryAmount()).multiply(new BigDecimal(1000)).toString());
                        liKuResourcePool.setLengthOfCable(liKuResourcePool.getInventoryAmount());
                        liKuResourcePool.setRemaining(liKuResourcePool.getInventoryAmount());
                    }
                    liKuIsSegmentPoolList.add(liKuResourcePool);
                }
            } else {
                liKuIsNotSegmentPoolList.add(liKuResourcePool);
            }
        });

        //排序    地区-》库龄降序-》数量大小降序-》批次号-》物料编码升序
        List<LiKuResourcePool> liKuIsSegmentPoolSortList = getSortLiKuPool(liKuIsSegmentPoolList);
        List<LiKuResourcePool> liKuIsNotSegmentPoolSortList = getSortLiKuPool(liKuIsNotSegmentPoolList);



        /*liKuIsNotSegmentPoolSortList.forEach(l -> {
            System.out.println(l.getLiKuFlag().getArea() + "---" + l.getMonthStorage() + "---" + l.getDaysStorage() + "---" + l.getInventoryAmount() + "---" + l.getBatch() + "---" + l.getExtendedCoding());
        });*/


        /*************************************************物资上报明细***************************************/
        //物资上报 段长物资
        List<MaterialReportDetails> materialIsSegmentList = new ArrayList<>();
        //物资上报 非段长物资
        List<MaterialReportDetails> materialIsNotSegmentList = new ArrayList<>();
        //根据 段长非段长区分 初始化
        materialReportDetailsList.forEach(l -> {
            //初始化利库数量
            l.setNumberOfNiches(0.0);
            //初始化申请数量
            l.setNumberOfApplications(l.getTotal());

            //单位转换为米
            if (l.getUnit().equals(MaterialsUnitFlagEnum.KM.getUnit())) {
                l.setTotal((BigDecimal.valueOf(l.getTotal()).multiply(new BigDecimal(1000))).doubleValue());
                l.setUnit(MaterialsUnitFlagEnum.M.getUnit());
                l.setNumberOfApplications(l.getTotal());
            }
            if (l.getUnit().equals(MaterialsUnitFlagEnum.M.getUnit())) {
                materialIsSegmentList.add(l);
            } else {
                materialIsNotSegmentList.add(l);
            }
        });

        /********************************************物资利库比较***************************************************/

        //非段长物资比较 本地库 物资匹配利库
        materialIsNotSegmentList.forEach(l -> {
            //优先匹配最佳
            liKuIsNotSegmentPoolSortList.stream().filter(liKu -> liKu.getLiKuFlag().getArea().equals(l.getReportingArea())
                    && Double.valueOf(liKu.getRemaining()).compareTo(l.getTotal()) >= 0
                    && l.getMaterialCode().equals(liKu.getMaterialCode())
                    && l.getExtendedCoding().equals(liKu.getExtendedCoding())).forEach(liKuResourcePool -> {
                liKuCompareMaterial(l, liKuResourcePool, AreaTypeFlagEnum.LOCAL, null, MaterialsUnitFlagEnum.IS_SEGMENT);
            });
            if (l.getTotal().compareTo(l.getNumberOfNiches()) != 0) {
                //相同地区进行比较
                liKuIsNotSegmentPoolSortList.stream().filter(liKu -> liKu.getLiKuFlag().getArea().equals(l.getReportingArea())
                        && l.getMaterialCode().equals(liKu.getMaterialCode())
                        && l.getExtendedCoding().equals(liKu.getExtendedCoding())).forEach(liKuResourcePool -> {
                    liKuCompareMaterial(l, liKuResourcePool, AreaTypeFlagEnum.LOCAL, null, MaterialsUnitFlagEnum.IS_SEGMENT);
                });
            }
        });
        //非段长物资比较 跨区库
        materialIsNotSegmentList.forEach(l -> {
            //需求数量与利库数量不同 去其他地区找
            if (l.getTotal().compareTo(l.getNumberOfNiches()) != 0) {
                //优先匹配最佳
                liKuIsNotSegmentPoolSortList.stream().filter(liKu -> !liKu.getLiKuFlag().getArea().equals(l.getReportingArea())
                        && Double.valueOf(liKu.getRemaining()).compareTo(l.getTotal()) >= 0
                        && l.getMaterialCode().equals(liKu.getMaterialCode())
                        && l.getExtendedCoding().equals(liKu.getExtendedCoding())).forEach(liKuResourcePool -> {
                    liKuCompareMaterial(l, liKuResourcePool, AreaTypeFlagEnum.SPAN, null, MaterialsUnitFlagEnum.IS_SEGMENT);
                });
            }
            if (l.getTotal().compareTo(l.getNumberOfNiches()) != 0) {
                //不同地区比较
                liKuIsNotSegmentPoolSortList.stream().filter(liKu -> !liKu.getLiKuFlag().getArea().equals(l.getReportingArea())
                        && l.getMaterialCode().equals(liKu.getMaterialCode())
                        && l.getExtendedCoding().equals(liKu.getExtendedCoding())).forEach(liKuResourcePool -> {
                    liKuCompareMaterial(l, liKuResourcePool, AreaTypeFlagEnum.SPAN, null, MaterialsUnitFlagEnum.IS_SEGMENT);
                });
            }
        });
        //非段长物资替代 本地库
        materialIsNotSegmentList.forEach(l -> {
            //如果本地库 区域库依旧没找到 则启用代替
            if (l.getTotal().compareTo(l.getNumberOfNiches()) != 0) {
                //过滤出 本地库物料编码相同 最优选择
                liKuIsNotSegmentPoolSortList.stream().filter(liKu -> liKu.getLiKuFlag().getArea().equals(l.getReportingArea())
                        && Double.valueOf(liKu.getRemaining()).compareTo(l.getTotal()) >= 0
                        && l.getMaterialCode().equals(liKu.getMaterialCode())).forEach(liKuResourcePool -> {
                    liKuCompareMaterial(l, liKuResourcePool, AreaTypeFlagEnum.LOCAL, AreaTypeFlagEnum.LOCAL_REPLACEMENT, MaterialsUnitFlagEnum.IS_SEGMENT);
                });
                if (l.getTotal().compareTo(l.getNumberOfNiches()) != 0) {
                    liKuIsNotSegmentPoolSortList.stream().filter(liKu -> liKu.getLiKuFlag().getArea().equals(l.getReportingArea())
                            && l.getMaterialCode().equals(liKu.getMaterialCode())).forEach(liKuResourcePool -> {
                        liKuCompareMaterial(l, liKuResourcePool, AreaTypeFlagEnum.LOCAL, AreaTypeFlagEnum.LOCAL_REPLACEMENT, MaterialsUnitFlagEnum.IS_SEGMENT);
                    });
                }
            }
        });
        //非段长物资代替 库区库
        materialIsNotSegmentList.forEach(l -> {
            //如果本地库 区域库依旧没找到 则启用代替
            if (l.getTotal().compareTo(l.getNumberOfNiches()) != 0) {
                //过滤出 本地库物料编码相同 最优选择
                liKuIsNotSegmentPoolSortList.stream().filter(liKu -> !liKu.getLiKuFlag().getArea().equals(l.getReportingArea())
                        && Double.valueOf(liKu.getRemaining()).compareTo(l.getTotal()) >= 0
                        && l.getMaterialCode().equals(liKu.getMaterialCode())).forEach(liKuResourcePool -> {
                    liKuCompareMaterial(l, liKuResourcePool, AreaTypeFlagEnum.SPAN, AreaTypeFlagEnum.SPAN_REPLACEMENT, MaterialsUnitFlagEnum.IS_SEGMENT);
                });
                if (l.getTotal().compareTo(l.getNumberOfNiches()) != 0) {
                    liKuIsNotSegmentPoolSortList.stream().filter(liKu -> !liKu.getLiKuFlag().getArea().equals(l.getReportingArea())
                            && l.getMaterialCode().equals(liKu.getMaterialCode())).forEach(liKuResourcePool -> {
                        liKuCompareMaterial(l, liKuResourcePool, AreaTypeFlagEnum.SPAN, AreaTypeFlagEnum.SPAN_REPLACEMENT, MaterialsUnitFlagEnum.IS_SEGMENT);
                    });
                }
            }
        });


        //段长物资比较 本地库 利库匹配物资
        liKuIsSegmentPoolSortList.forEach(l -> {
            //最优选择 过滤本地库  物料编码  扩展编码 段长大于上报物资
            materialIsSegmentList.stream().filter(material -> material.getReportingArea().equals(l.getLiKuFlag().getArea())
                    && material.getMaterialCode().equals(l.getMaterialCode())
                    && material.getExtendedCoding().equals(l.getExtendedCoding())
                    && new BigDecimal(l.getLengthOfCable()).compareTo(BigDecimal.valueOf(material.getTotal())) == 0).forEach(materialReportDetails -> {
                        if (new BigDecimal(l.getRemaining()).compareTo(BigDecimal.ZERO) > 0) {
                            liKuCompareMaterial(materialReportDetails, l, AreaTypeFlagEnum.LOCAL, null, MaterialsUnitFlagEnum.IS_NOT_SEGMENT);
                        }
                    }
            );
            if (new BigDecimal(l.getRemaining()).compareTo(BigDecimal.ZERO) > 0) {
                materialIsSegmentList.stream().filter(material -> material.getReportingArea().equals(l.getLiKuFlag().getArea())
                        && material.getMaterialCode().equals(l.getMaterialCode())
                        && material.getExtendedCoding().equals(l.getExtendedCoding())
                        && new BigDecimal(l.getLengthOfCable()).compareTo(BigDecimal.valueOf(material.getTotal())) > 0).forEach(materialReportDetails -> {
                            if (new BigDecimal(l.getRemaining()).compareTo(BigDecimal.valueOf(materialReportDetails.getTotal())) >= 0) {
                                liKuCompareMaterial(materialReportDetails, l, AreaTypeFlagEnum.LOCAL, null, MaterialsUnitFlagEnum.IS_NOT_SEGMENT);
                            }
                        }
                );
            }
        });
        //段长物资比较 跨区库
        liKuIsSegmentPoolSortList.forEach(l -> {
            //最优选择 过滤本地库  物料编码  扩展编码 段长大于上报物资
            materialIsSegmentList.stream().filter(material -> !material.getReportingArea().equals(l.getLiKuFlag().getArea())
                    && material.getMaterialCode().equals(l.getMaterialCode())
                    && material.getExtendedCoding().equals(l.getExtendedCoding())
                    && new BigDecimal(l.getLengthOfCable()).compareTo(BigDecimal.valueOf(material.getTotal())) == 0).forEach(materialReportDetails -> {
                        if (new BigDecimal(l.getRemaining()).compareTo(BigDecimal.ZERO) > 0) {
                            liKuCompareMaterial(materialReportDetails, l, AreaTypeFlagEnum.SPAN, null, MaterialsUnitFlagEnum.IS_NOT_SEGMENT);
                        }
                    }
            );
            if (new BigDecimal(l.getRemaining()).compareTo(BigDecimal.ZERO) > 0) {
                materialIsSegmentList.stream().filter(material -> !material.getReportingArea().equals(l.getLiKuFlag().getArea())
                        && material.getMaterialCode().equals(l.getMaterialCode())
                        && material.getExtendedCoding().equals(l.getExtendedCoding())
                        && new BigDecimal(l.getLengthOfCable()).compareTo(BigDecimal.valueOf(material.getTotal())) > 0).forEach(materialReportDetails -> {
                            if (new BigDecimal(l.getRemaining()).compareTo(BigDecimal.valueOf(materialReportDetails.getTotal())) >= 0) {
                                liKuCompareMaterial(materialReportDetails, l, AreaTypeFlagEnum.SPAN, null, MaterialsUnitFlagEnum.IS_NOT_SEGMENT);
                            }
                        }
                );
            }
        });
        //段长物资替换 本地库
        liKuIsSegmentPoolSortList.forEach(l -> {
            //最优选择 过滤本地库  物料编码  扩展编码 段长大于上报物资
            materialIsSegmentList.stream().filter(material -> material.getReportingArea().equals(l.getLiKuFlag().getArea())
                    && material.getMaterialCode().equals(l.getMaterialCode())
                    && new BigDecimal(l.getLengthOfCable()).compareTo(BigDecimal.valueOf(material.getTotal())) == 0).forEach(materialReportDetails -> {
                        if (new BigDecimal(l.getRemaining()).compareTo(BigDecimal.ZERO) > 0) {
                            liKuCompareMaterial(materialReportDetails, l, AreaTypeFlagEnum.LOCAL, AreaTypeFlagEnum.LOCAL_REPLACEMENT, MaterialsUnitFlagEnum.IS_NOT_SEGMENT);
                        }
                    }
            );
            if (new BigDecimal(l.getRemaining()).compareTo(BigDecimal.ZERO) > 0) {
                materialIsSegmentList.stream().filter(material -> material.getReportingArea().equals(l.getLiKuFlag().getArea())
                        && material.getMaterialCode().equals(l.getMaterialCode())
                        && new BigDecimal(l.getLengthOfCable()).compareTo(BigDecimal.valueOf(material.getTotal())) > 0).forEach(materialReportDetails -> {
                            if (new BigDecimal(l.getRemaining()).compareTo(BigDecimal.valueOf(materialReportDetails.getTotal())) >= 0) {
                                liKuCompareMaterial(materialReportDetails, l, AreaTypeFlagEnum.LOCAL, AreaTypeFlagEnum.LOCAL_REPLACEMENT, MaterialsUnitFlagEnum.IS_NOT_SEGMENT);
                            }
                        }
                );
            }
        });
        //段长物资替换 跨区库
        liKuIsSegmentPoolSortList.forEach(l -> {
            //最优选择 过滤本地库  物料编码  扩展编码 段长大于上报物资
            materialIsSegmentList.stream().filter(material -> !material.getReportingArea().equals(l.getLiKuFlag().getArea())
                    && material.getMaterialCode().equals(l.getMaterialCode())
                    && new BigDecimal(l.getLengthOfCable()).compareTo(BigDecimal.valueOf(material.getTotal())) == 0).forEach(materialReportDetails -> {
                        if (new BigDecimal(l.getRemaining()).compareTo(BigDecimal.ZERO) > 0) {
                            liKuCompareMaterial(materialReportDetails, l, AreaTypeFlagEnum.SPAN, AreaTypeFlagEnum.SPAN_REPLACEMENT, MaterialsUnitFlagEnum.IS_NOT_SEGMENT);
                        }
                    }
            );
            if (new BigDecimal(l.getRemaining()).compareTo(BigDecimal.ZERO) > 0) {
                materialIsSegmentList.stream().filter(material -> !material.getReportingArea().equals(l.getLiKuFlag().getArea())
                        && material.getMaterialCode().equals(l.getMaterialCode())
                        && new BigDecimal(l.getLengthOfCable()).compareTo(BigDecimal.valueOf(material.getTotal())) > 0).forEach(materialReportDetails -> {
                            if (new BigDecimal(l.getRemaining()).compareTo(BigDecimal.valueOf(materialReportDetails.getTotal())) >= 0) {
                                liKuCompareMaterial(materialReportDetails, l, AreaTypeFlagEnum.SPAN, AreaTypeFlagEnum.SPAN_REPLACEMENT, MaterialsUnitFlagEnum.IS_NOT_SEGMENT);
                            }
                        }
                );
            }
        });

        //段长非段长合并 排序 处理拆分段长
        List<LiKuResourcePool> tempList = new ArrayList<>();
        liKuIsSegmentPoolSortList.stream().map(m -> {
            m.setLengthOfCable(m.getRemaining() + ";");

            return m;
        }).forEach(l -> {
            if (tempList.size() != 0) {
                boolean flag = true;
                for (LiKuResourcePool temp : tempList) {
                    if (l.getExtendedCoding().equals(temp.getExtendedCoding())
                            && l.getMaterialCode().equals(temp.getMaterialCode())
                            && l.getInventoryAmount().equals(temp.getInventoryAmount())
                            && l.getInvLocation().equals(temp.getInvLocation())
                            && l.getBatch().equals(temp.getBatch())
                            && l.getFactory().equals(temp.getFactory())
                            && l.getPlanner().equals(temp.getPlanner())) {
                        //段长拼接
                        temp.setLengthOfCable(temp.getLengthOfCable() + l.getRemaining() + ";");
                        //修改 剩余总量
                        temp.setRemaining((new BigDecimal(temp.getRemaining()).add(new BigDecimal(l.getRemaining()))).toString());
                        //修改 利库数量
                        temp.setNumberOfNiches((new BigDecimal(temp.getNumberOfNiches()).add(new BigDecimal(l.getNumberOfNiches()))).toString());
                        flag = false;

                        break;
                    }
                }
                if (flag) {
                    tempList.add(l);
                }
            } else {
                tempList.add(l);
            }
        });
        //利库合并 并排序
        List<LiKuResourcePool> sortLiKuPoolList = getSortLiKuPool(Stream.concat(liKuIsNotSegmentPoolSortList.stream(), tempList.stream()).map(m -> {
            m.setLengthOfCable(interceptString(m.getLengthOfCable()));
            return m;
        }).collect(Collectors.toList()));
        //上报资源合并
        List<MaterialReportDetails> materialResultList = Stream.concat(materialIsNotSegmentList.stream(), materialIsSegmentList.stream()).map(m -> {
            //处理上报资源中多余，
            m.setLocalWareBatch(interceptString(m.getLocalWareBatch()));
            m.setLocalWareHouse(interceptString(m.getLocalWareHouse()));
            m.setNumberOfLocal(interceptString(m.getNumberOfLocal()));

            m.setRegionalBatch(interceptString(m.getRegionalBatch()));
            m.setRegionalHouse(interceptString(m.getRegionalHouse()));
            m.setNumberOfRegional(interceptString(m.getNumberOfRegional()));

            m.setLocalReplacementExtendedCoding(interceptString(m.getLocalReplacementExtendedCoding()));
            m.setLocalReplacementExtendedDescription(interceptString(m.getLocalReplacementExtendedDescription()));
            m.setLocalReplacementBatch(interceptString(m.getLocalReplacementBatch()));
            m.setLocalReplacementHouse(interceptString(m.getLocalReplacementHouse()));
            m.setLocalReplacementNumber(interceptString(m.getLocalReplacementNumber()));

            m.setRegionalReplacementExtendedCoding(interceptString(m.getRegionalReplacementExtendedCoding()));
            m.setRegionalReplacementExtendedDescription(interceptString(m.getRegionalReplacementExtendedDescription()));
            m.setRegionalReplacementBatch(interceptString(m.getRegionalReplacementBatch()));
            m.setRegionalReplacementHouse(interceptString(m.getRegionalReplacementHouse()));
            m.setRegionalReplacementNumber(interceptString(m.getRegionalReplacementNumber()));

            return m;
        }).collect(Collectors.toList());

        /***************************************存储数据库************************************************/
        //清除利库池
        liKuResourcePoolMapper.clearLiKuResource();
        materialReportDetailsMapper.delAll();

        sortLiKuPoolList.forEach(DataEntity::preInsert);
        liKuResourcePoolMapper.insertList(sortLiKuPoolList);

        materialResultList.forEach(DataEntity::preInsert);
        materialReportDetailsMapper.insertList(materialResultList);
        j.setSuccess(true);
        j.setMsg("已成功导入 " + materialResultList.size() + " 条物资上报明细记录");

        return j;
    }

    /**
     * 处理多余字符串
     */
    private String interceptString(String str) {
        if (StringUtils.isNotBlank(str)) {
            return str.substring(0, str.length() - 1);
        }
        return null;
    }

    /**
     * 计算处理库
     *
     * @param materialReportDetails 上报资源
     * @param liKuResourcePool      利库资源池
     * @param areaTypeFlagEnum      地区类型
     * @param replacement           地区替换类型
     */
    private void liKuCompareMaterial(MaterialReportDetails materialReportDetails, LiKuResourcePool liKuResourcePool, AreaTypeFlagEnum areaTypeFlagEnum, AreaTypeFlagEnum replacement, MaterialsUnitFlagEnum judgingSegment) {
        //如果相等直接结束方法 利库数+总数
        if (materialReportDetails.getTotal().compareTo(materialReportDetails.getNumberOfNiches()) == 0 || new BigDecimal(liKuResourcePool.getRemaining()).compareTo(BigDecimal.ZERO) == 0) {
            return;
        }
        //如果物料编码与扩展编码相同，说明是需要的物资
        //利库池 利库数
        BigDecimal liKu_liKuCount = new BigDecimal(liKuResourcePool.getNumberOfNiches());
        //利库池剩余 量
        BigDecimal liKu_remaining = new BigDecimal(liKuResourcePool.getRemaining());

        //上报物资 申请数量
        BigDecimal material_ApplicationCount = BigDecimal.valueOf(materialReportDetails.getNumberOfApplications());
        //上报物资利库量
        BigDecimal material_liKuCount = BigDecimal.valueOf(materialReportDetails.getNumberOfNiches());

        //临时变量
        BigDecimal temp = new BigDecimal(0);
        if (material_ApplicationCount.compareTo(liKu_remaining) > 0) {
            temp = liKu_remaining;//可能利库池剩余数量不够
        } else {
            temp = material_ApplicationCount;//利库池数量满足 上报需求
        }

        //利库池 剩余量
        liKuResourcePool.setRemaining(liKu_remaining.subtract(temp).toString());
        //利库量
        liKuResourcePool.setNumberOfNiches(new BigDecimal(liKuResourcePool.getNumberOfNiches()).add(temp).toString());
        //物资  申请数量
        materialReportDetails.setNumberOfApplications(material_ApplicationCount.subtract(temp).doubleValue());
        //物资  利库量
        //if (judgingSegment == MaterialsUnitFlagEnum.IS_NOT_SEGMENT) {
            materialReportDetails.setNumberOfNiches(material_liKuCount.add(temp).doubleValue());
        /*} else {
            materialReportDetails.setNumberOfNiches(material_liKuCount.add(temp).doubleValue());
        }*/

        //如果代替标识为空则
        if (replacement == null) {
            //传递标识，根据标识来判断本地/跨库 所有结束后统一处理多余，
            if (AreaTypeFlagEnum.LOCAL == areaTypeFlagEnum) {
                //拼接本地库批次号
                materialReportDetails.setLocalWareBatch(materialReportDetails.getLocalWareBatch() + liKuResourcePool.getBatch() + ",");
                materialReportDetails.setLocalWareHouse(materialReportDetails.getLocalWareHouse() + liKuResourcePool.getInvLocation() + ",");
                materialReportDetails.setNumberOfLocal(materialReportDetails.getNumberOfLocal() + temp + ",");
            } else {
                //拼接跨区批次号
                materialReportDetails.setRegionalBatch(materialReportDetails.getRegionalBatch() + liKuResourcePool.getBatch() + ",");
                materialReportDetails.setRegionalHouse(materialReportDetails.getRegionalHouse() + liKuResourcePool.getInvLocation() + ",");
                materialReportDetails.setNumberOfRegional(materialReportDetails.getNumberOfRegional() + temp + ",");
            }
        } else {
            //区分本地利库与跨区利库替换
            if (AreaTypeFlagEnum.LOCAL_REPLACEMENT == replacement) {
                materialReportDetails.setLocalReplacementExtendedCoding(materialReportDetails.getLocalReplacementExtendedCoding() + liKuResourcePool.getExtendedCoding() + ",");
                materialReportDetails.setLocalReplacementExtendedDescription(materialReportDetails.getLocalReplacementExtendedDescription() + liKuResourcePool.getExtendedDescription() + ",");
                materialReportDetails.setLocalReplacementBatch(materialReportDetails.getLocalReplacementBatch() + liKuResourcePool.getBatch() + ",");
                materialReportDetails.setLocalReplacementHouse(materialReportDetails.getLocalReplacementHouse() + liKuResourcePool.getInvLocation() + ",");
                materialReportDetails.setLocalReplacementNumber(materialReportDetails.getLocalReplacementNumber() + temp + ",");
            } else {
                materialReportDetails.setRegionalReplacementExtendedCoding(materialReportDetails.getRegionalReplacementExtendedCoding() + liKuResourcePool.getExtendedCoding() + ",");
                materialReportDetails.setRegionalReplacementExtendedDescription(materialReportDetails.getRegionalReplacementExtendedDescription() + liKuResourcePool.getExtendedDescription() + ",");
                materialReportDetails.setRegionalReplacementBatch(materialReportDetails.getRegionalReplacementBatch() + liKuResourcePool.getBatch() + ",");
                materialReportDetails.setRegionalReplacementHouse(materialReportDetails.getRegionalReplacementHouse() + liKuResourcePool.getInvLocation() + ",");
                materialReportDetails.setRegionalReplacementNumber(materialReportDetails.getRegionalReplacementNumber() + temp + ",");
            }
        }

    }

    /**
     * 排序 地区-》库龄降序-》数量大小降序-》批次号-》物料编码升序
     *
     * @param liKuPoolList 利库池
     * @return
     */
    private List<LiKuResourcePool> getSortLiKuPool(List<LiKuResourcePool> liKuPoolList) {
        return liKuPoolList.stream().sorted((liKuSort1, liKuSort2) -> {
            if (liKuSort1.getLiKuFlag().getNum().compareTo(liKuSort2.getLiKuFlag().getNum()) == 0) {
                //库龄按照月排序
                if (liKuSort1.getMonthStorage().compareTo(liKuSort2.getMonthStorage()) == 0) {
                    //库龄按照日降序
                    if (liKuSort1.getDaysStorage().compareTo(liKuSort2.getDaysStorage()) == 0) {
                        //数量按大小降序
                        if (new BigDecimal(liKuSort1.getInventoryAmount()).compareTo(new BigDecimal(liKuSort2.getInventoryAmount())) == 0) {
                            //批次号自然顺序
                            if (liKuSort1.getBatch().compareTo(liKuSort2.getBatch()) == 0) {
                                //物料编码升序
                                return new BigDecimal(liKuSort1.getExtendedCoding()).compareTo(new BigDecimal(liKuSort2.getExtendedCoding()));
                            } else {
                                return liKuSort1.getBatch().compareTo(liKuSort2.getBatch());
                            }
                        } else {
                            return new BigDecimal(liKuSort2.getInventoryAmount()).compareTo(new BigDecimal(liKuSort1.getInventoryAmount()));
                        }
                    } else {
                        return liKuSort2.getDaysStorage() - liKuSort1.getDaysStorage();
                    }
                } else {
                    return liKuSort2.getMonthStorage() - (liKuSort1.getMonthStorage());
                }
            } else {
                return liKuSort1.getLiKuFlag().getNum() - liKuSort2.getLiKuFlag().getNum();
            }
        }).collect(Collectors.toList());

    }

}