/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.sg.balancedlibrary.materialReportDetails.web;

import com.google.common.collect.Lists;
import com.jeeplus.common.json.AjaxJson;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.core.persistence.Page;
import com.jeeplus.core.web.BaseController;
import com.jeeplus.modules.sg.balancedlibrary.liKuResourcePool.entity.LiKuResourcePool;
import com.jeeplus.modules.sg.balancedlibrary.liKuResourcePool.service.LiKuResourcePoolService;
import com.jeeplus.modules.sg.balancedlibrary.liKuResourcePool.utils.LiKuUtils;
import com.jeeplus.modules.sg.balancedlibrary.materialQuota.entity.MaterialQuota;
import com.jeeplus.modules.sg.balancedlibrary.materialQuota.service.MaterialQuotaService;
import com.jeeplus.modules.sg.balancedlibrary.materialReportDetails.entity.CompareHistory;
import com.jeeplus.modules.sg.balancedlibrary.materialReportDetails.entity.LikuDetail;
import com.jeeplus.modules.sg.balancedlibrary.materialReportDetails.entity.MaterialDetails;
import com.jeeplus.modules.sg.balancedlibrary.materialReportDetails.entity.MaterialReportDetails;
import com.jeeplus.modules.sg.balancedlibrary.materialReportDetails.service.CompareHistoryService;
import com.jeeplus.modules.sg.balancedlibrary.materialReportDetails.service.MaterialReportDetailsService;
import com.jeeplus.modules.sg.balancedlibrary.materialReportDetails.utils.AdjustUtil;
import com.jeeplus.modules.sg.balancedlibrary.materialReportDetails.utils.ExcelUtil;
import com.jeeplus.modules.sg.balancedlibrary.materialReportDetails.utils.ExportUtil;
import com.jeeplus.modules.sg.balancedlibrary.materialReportDetails.utils.ImportExcel;
import com.jeeplus.modules.sg.balancedlibrary.onPassageMaterials.entity.MaterialInformation;
import com.jeeplus.modules.sg.balancedlibrary.preferredMaterials.service.PreferredMaterialsService;
import com.jeeplus.modules.sg.balancedlibrary.replaceMaterial.entity.ReplaceMaterial;
import com.jeeplus.modules.sg.balancedlibrary.replaceMaterial.service.ReplaceMaterialService;
import com.jeeplus.modules.sg.balancedlibrary.standardMaterials.entity.StandardMaterials;
import com.jeeplus.modules.sg.balancedlibrary.standardMaterials.service.StandardMaterialsService;
import com.jeeplus.modules.sg.financial.erpcredit.util.ValidationUtils;
import com.jeeplus.modules.sg.managementcenter.materialproject.entity.MaterialProject;
import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.multipart.MultipartFile;

import javax.resource.spi.work.Work;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.io.*;
import java.math.BigDecimal;
import java.util.*;

import static java.math.BigDecimal.ROUND_HALF_UP;

/**
 * 物资上报明细Controller
 *
 * @author enford
 * @version 1.0
 */
@Controller
@RequestMapping(value = "${adminPath}/MaterialReportDetails")
public class MaterialReportDetailsControllerTest extends BaseController {

    @Autowired
    private MaterialReportDetailsService service;

    @Autowired
    private LiKuResourcePoolService poolService;

    @Autowired
    private ReplaceMaterialService replaceMaterialService;

    @Autowired
    private CompareHistoryService compareHistoryService;

    @Autowired
    private PreferredMaterialsService preferredMaterialsService;

    @Autowired
    private MaterialQuotaService materialQuotaService;

    @Autowired
    private StandardMaterialsService standardMaterialsService;

    @ModelAttribute
    public MaterialReportDetails get(@RequestParam(required = false) String id) {
        MaterialReportDetails entity = null;
        if (StringUtils.isNotBlank(id)) {
            entity = service.get(id);
        }
        if (entity == null) {
            entity = new MaterialReportDetails();
        }
        return entity;
    }

    /**
     * 物资上报明细列表页面
     */
    @RequiresPermissions("sg:balancedlibrary:MaterialReportDetails:list")
    @RequestMapping(value = "list")
    public String list(MaterialReportDetails entity, Model model) {
        model.addAttribute("entity", entity);
        return "modules/sg/balancedlibrary/materialReportDetails/MaterialReportDetailslList";
    }

    /**
     * 利库明细页面
     */
    @RequiresPermissions("sg:balancedlibrary:MaterialReportDetails:list")
    @RequestMapping(value = "likuResults")
    public String likuResults(MaterialReportDetails entity, Model model) {
        model.addAttribute("entity", entity);
        return "modules/sg/balancedlibrary/materialReportDetails/LikuResults";
    }

    /**
     * 利库明细数据
     */
    @ResponseBody
    @RequiresPermissions("sg:balancedlibrary:MaterialReportDetails:list")
    @RequestMapping(value = "likuResultsData")
    public Map<String, Object> likuResultsData(MaterialReportDetails entity, HttpServletRequest request, HttpServletResponse response, Model model) {
        List<LikuDetail> list = service.getLikuDetails(entity);
        Map<String, Object> map = new HashMap<String, Object>();
        map.put("rows", list);
        map.put("total", list.size());
        return map;
    }

    /**
     * 物资上报明细列表数据
     */
    @ResponseBody
    @RequiresPermissions("sg:balancedlibrary:MaterialReportDetails:list")
    @RequestMapping(value = "data")
    public Map<String, Object> data(MaterialReportDetails entity, HttpServletRequest request, HttpServletResponse response, Model model) {
        HttpSession session = request.getSession();
        session.setAttribute("materialReportDetails", entity);
        Page<MaterialReportDetails> page = service.findPage(new Page<MaterialReportDetails>(request, response), entity);
        return getBootstrapData(page);
    }

    /**
     * 查看，增加，编辑物资上报明细表单页面
     */
    @RequiresPermissions(value = {"sg:balancedlibrary:MaterialReportDetails:view", "sg:balancedlibrary:MaterialReportDetails:add", "sg:balancedlibrary:MaterialReportDetails:edit"}, logical = Logical.OR)
    @RequestMapping(value = "form")
    public String form(MaterialReportDetails entity, Model model) {
        model.addAttribute("entity", entity);
        return "modules/test/one/dialog/MaterialReportDetailsForm";
    }

    /**
     * 保存物资上报明细
     */
    @ResponseBody
    @RequiresPermissions(value = {"sg:balancedlibrary:MaterialReportDetails:add", "sg:balancedlibrary:MaterialReportDetails:edit"}, logical = Logical.OR)
    @RequestMapping(value = "save")
    public AjaxJson save(MaterialReportDetails entity, Model model) throws Exception {
        AjaxJson j = new AjaxJson();
        /**
         * 后台hibernate-validation插件校验
         */
        String errMsg = beanValidator(entity);
        if (StringUtils.isNotBlank(errMsg)) {
            j.setSuccess(false);
            j.setMsg(errMsg);
            return j;
        }
        //新增或编辑表单保存
        service.save(entity);
        j.setSuccess(true);
        j.setMsg("保存物资上报明细成功");
        return j;
    }

    /**
     * 删除物资上报明细
     */
    @ResponseBody
    @RequiresPermissions("sg:balancedlibrary:MaterialReportDetails:del")
    @RequestMapping(value = "delete")
    public AjaxJson delete(MaterialReportDetails entity) {
        AjaxJson j = new AjaxJson();
        service.delete(entity);
        j.setMsg("删除物资上报明细成功");
        return j;
    }

    /**
     * 批量删除物资上报明细
     */
    @ResponseBody
    @RequiresPermissions("sg:balancedlibrary:MaterialReportDetails:del")
    @RequestMapping(value = "deleteAll")
    public AjaxJson deleteAll(String ids) {
        AjaxJson j = new AjaxJson();
        String idArray[] = ids.split(",");
        for (String id : idArray) {
            service.delete(service.get(id));
        }
        j.setMsg("删除物资上报明细成功");
        return j;
    }

    /**
     * 导出excel文件
     */
    @ResponseBody
    @RequestMapping(value = "export")
    public void exportFile(HttpServletRequest request, HttpServletResponse response) {
        HttpSession session = request.getSession();
        MaterialReportDetails entity = (MaterialReportDetails) session.getAttribute("materialReportDetails");
        entity.setPage(null);
        List<MaterialReportDetails> reportList = service.findList(entity);
        List<LikuDetail> likuList = service.getLikuDetails(entity);
        ExportUtil util = new ExportUtil();
        util.exportDetails(response,reportList,likuList);
    }

    /**
     * 物资上报明细-导入Excel数据
     */
    @ResponseBody
    @RequestMapping(value = "import")
    public AjaxJson importFile(@RequestParam("file") MultipartFile file, HttpServletResponse response, HttpServletRequest request) {
        AjaxJson j = new AjaxJson();
        try {
            ImportExcel ei = new ImportExcel(file, 1, 0);
            //物资上报明细数据列表
            List<MaterialReportDetails> list = ei.getDataList(MaterialReportDetails.class);

            //如果没有利库资源池数据 直接返回
            j = service.liKuBalance(list);
            if(!j.isSuccess()){
                return j;
            }


            String fileName = "项目物资需求表" + DateUtils.getDate("yyyyMMddHHmmss") + ".xlsx";
            new ExportExcel("", MaterialReportDetails.class).setDataList(list).write(response, fileName).dispose();


            //利库基础数据更新库存
            //poolService.updateAllList(poolList);

            /*if (list.size() > 0) {
                service.delAll();
                service.insertList(list);
            }
            */
        } catch (Exception e) {
            j.setSuccess(false);
            j.setMsg("导入物资上报明细失败！失败信息：" + e.getMessage());
            e.printStackTrace();


        }
        return j;
    }



    /**
     * 利库非段长排序方法，排序依据 库存日、库存数量、批次号
     * @param bBNonPoolList
     */
    private void poolSort(List<LiKuResourcePool> bBNonPoolList) {
        try {
            Comparator<LiKuResourcePool> reversed = Comparator.comparing(LiKuResourcePool::getDaysStorage).reversed();
            Comparator<LiKuResourcePool> comparator = Comparator.comparing(LiKuResourcePool::getInventoryAmount).reversed();
            Comparator<LiKuResourcePool> poolComparator = Comparator.comparing(LiKuResourcePool::getBatch);
            bBNonPoolList.stream().sorted(reversed.thenComparing(comparator).thenComparing(poolComparator));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 利库段长排序方法，排序依据 库存日、批次号、具体段长
     * @param bBNonPoolList
     */
    private void poolSegSort(List<LiKuResourcePool> bBNonPoolList) {
        try {
            Comparator<LiKuResourcePool> reversed = Comparator.comparing(LiKuResourcePool::getDaysStorage).reversed();
            Comparator<LiKuResourcePool> comparator = Comparator.comparing(LiKuResourcePool::getInventoryAmount).reversed();
            Comparator<LiKuResourcePool> poolComparator = Comparator.comparing(LiKuResourcePool::getBatch);
            Comparator<LiKuResourcePool> beComparator = Comparator.comparing(LiKuResourcePool::getLengthOfCable);
            bBNonPoolList.stream().sorted(reversed.thenComparing(poolComparator).thenComparing(beComparator));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 下载物资上报明细导入模板
     */
    @ResponseBody
    @RequestMapping(value = "import/template")
    public AjaxJson importFileTemplate(HttpServletResponse response) {
        AjaxJson j = new AjaxJson();
        try {
            String fileName = "物资信息.xlsx";
            List<MaterialReportDetails> list = Lists.newArrayList();
            new ExportExcel("物资信息表", MaterialReportDetails.class, 1).setDataList(list).write(response, fileName).dispose();
            return null;
        } catch (Exception e) {
            j.setSuccess(false);
            j.setMsg("导入物资信息模板下载失败！失败信息：" + e.getMessage());
        }
        return j;
    }

    /**
     * 下载物资上报明细导入模板 新的直接从模板文件中获取
     */
    @ResponseBody
    @RequestMapping(value = "import/templateNew")
    public void  importFileTemplateNew(HttpServletResponse response) throws IOException {
        String fileName = "项目物资需求.xlsx";
        String path = this.getClass().getResource("/").getPath()+ "freemarker/excelmodel/requiredMaterialModel.xlsx";
        File file = new File(path);
        InputStream inputStream = new FileInputStream(file);// 将excel文件转为输入流
        XSSFWorkbook workbook = new XSSFWorkbook(inputStream);// 创建个workbook，
        ExportUtil.exportFile(response,fileName,workbook);
    }

    /**
     * 对项目需求物资做验证
     */
    @ResponseBody
    @RequestMapping(value = "importVerify")
    public AjaxJson importFileAndVerify(@RequestParam("file") MultipartFile file, HttpServletResponse response, HttpServletRequest request) throws IOException, InvalidFormatException {
        AjaxJson j = new AjaxJson();
        try {
            ExcelUtil util = new ExcelUtil(file,1,0);

            List<StandardMaterials> standardList = standardMaterialsService.findAllList(new StandardMaterials());
            List<String> preferredCodes = preferredMaterialsService.findAllCodes();
            List<MaterialQuota> quotaList = materialQuotaService.findAllList(new MaterialQuota());
            util.dealVerify(standardList,preferredCodes,quotaList);

            HttpSession session = request.getSession();
            session.setAttribute(ValidationUtils.REQUIRED_MATERIAL_VERIFY,file.getOriginalFilename());
            util.saveToServer(session.getId());
//
//            ValidationUtils.storeObjToRedis(request,workbook,ValidationUtils.REQUIRED_MATERIAL_VERIFY);

            j.setSuccess(true);
            j.setMsg("导入成功，马上下载！");
            return j;
        } catch (Exception e) {
            j.setSuccess(false);
            j.setMsg("导入项目需求物资失败！失败信息：" + e.getMessage());
        }
        return j;


    }

    @ResponseBody
    @RequestMapping(value = "exportVerify")
    public void exportVerify(HttpServletResponse response, HttpServletRequest request) throws IOException, InvalidFormatException {
//        ExcelUtil util = new ExcelUtil(file,1,0);

//        Workbook workbook = ValidationUtils.getSessionRedisObj(request,ValidationUtils.REQUIRED_MATERIAL_VERIFY,Workbook.class);
        HttpSession session = request.getSession();
        String  fileName = (String)session.getAttribute(ValidationUtils.REQUIRED_MATERIAL_VERIFY);
        ExcelUtil util = new ExcelUtil();
        util.exportFile(response,fileName,session.getId());



    }


}